@@ -16,8 +16,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -39,7 +39,6 @@ exception statement from your version. */
 
 package java.io;
 
-import gnu.classpath.Configuration;
 import gnu.java.io.ObjectIdentityWrapper;
 
 import java.lang.reflect.Array;
@@ -53,6 +52,8 @@ import java.security.AccessController;
 import java.security.PrivilegedAction;
 import java.util.Arrays;
 import java.util.Hashtable;
+import java.util.Iterator;
+import java.util.TreeSet;
 import java.util.Vector;
 
 public class ObjectInputStream extends InputStream
@@ -75,7 +76,7 @@ public class ObjectInputStream extends InputStream
   public ObjectInputStream(InputStream in)
     throws IOException, StreamCorruptedException
   {
-    if (Configuration.DEBUG)
+    if (DEBUG)
       {
 	String val = System.getProperty("gcj.dumpobjects");
 	if (dump == false && val != null && !val.equals(""))
@@ -91,7 +92,6 @@ public class ObjectInputStream extends InputStream
       }
 
     this.resolveEnabled = false;
-    this.isDeserializing = false;
     this.blockDataPosition = 0;
     this.blockDataBytes = 0;
     this.blockData = new byte[BUFFER_SIZE];
@@ -99,7 +99,6 @@ public class ObjectInputStream extends InputStream
     this.realInputStream = new DataInputStream(in);
     this.nextOID = baseWireHandle;
     this.objectLookupTable = new Hashtable();
-    this.validators = new Vector();
     this.classLookupTable = new Hashtable();
     setBlockDataMode(true);
     readStreamHeader();
@@ -121,294 +120,315 @@ public class ObjectInputStream extends InputStream
    * @exception IOException Exception from underlying
    * <code>InputStream</code>.
    */
-  public final Object readObject() throws ClassNotFoundException, IOException
+  public final Object readObject()
+    throws ClassNotFoundException, IOException
   {
     if (this.useSubclassMethod)
       return readObjectOverride();
 
-    boolean was_deserializing;
-
     Object ret_val;
-    was_deserializing = this.isDeserializing;
-
-    boolean is_consumed = false;
     boolean old_mode = setBlockDataMode(false);
-
-    this.isDeserializing = true;
-
     byte marker = this.realInputStream.readByte();
 
-    depth += 2;
+    if (DEBUG)
+      depth += 2;
 
     if(dump) dumpElement("MARKER: 0x" + Integer.toHexString(marker) + " ");
 
     try
       {
-	switch (marker)
-	  {
-	  case TC_ENDBLOCKDATA:
-	    {
-	      ret_val = null;
-	      is_consumed = true;
-	      break;
-	    }
-
-	  case TC_BLOCKDATA:
-	  case TC_BLOCKDATALONG:
-	    {
-	      if (marker == TC_BLOCKDATALONG)
-		{ if(dump) dumpElementln("BLOCKDATALONG"); }
-	      else
-		{ if(dump) dumpElementln("BLOCKDATA"); }
-	      readNextBlock(marker);
-	      throw new StreamCorruptedException("Unexpected blockData");
-	    }
-
-	  case TC_NULL:
-	    {
-	      if(dump) dumpElementln("NULL");
-	      ret_val = null;
-	      break;
-	    }
-
-	  case TC_REFERENCE:
-	    {
-	      if(dump) dumpElement("REFERENCE ");
-	      Integer oid = new Integer(this.realInputStream.readInt());
-	      if(dump) dumpElementln(Integer.toHexString(oid.intValue()));
-	      ret_val = ((ObjectIdentityWrapper)
-			 this.objectLookupTable.get(oid)).object;
-	      break;
-	    }
-
-	  case TC_CLASS:
-	    {
-	      if(dump) dumpElementln("CLASS");
-	      ObjectStreamClass osc = (ObjectStreamClass)readObject();
-	      Class clazz = osc.forClass();
-	      assignNewHandle(clazz);
-	      ret_val = clazz;
-	      break;
-	    }
-
-	  case TC_PROXYCLASSDESC:
-	    {
-	      if(dump) dumpElementln("PROXYCLASS");
-	      int n_intf = this.realInputStream.readInt();
-	      String[] intfs = new String[n_intf];
-	      for (int i = 0; i < n_intf; i++)
-		{
-		  intfs[i] = this.realInputStream.readUTF();
-		  System.out.println(intfs[i]);
-		}
-	      
-	      boolean oldmode = setBlockDataMode(true);
-	      Class cl = resolveProxyClass(intfs);
-	      setBlockDataMode(oldmode);
-	      
-	      ObjectStreamClass osc = lookupClass(cl);
-	      assignNewHandle(osc);
-	      
-	      if (!is_consumed)
-		{
-		  byte b = this.realInputStream.readByte();
-		  if (b != TC_ENDBLOCKDATA)
-		    throw new IOException("Data annotated to class was not consumed." + b);
-		}
-	      else
-		is_consumed = false;
-	      ObjectStreamClass superosc = (ObjectStreamClass)readObject();
-	      osc.setSuperclass(superosc);
-	      ret_val = osc;
-	      break;
-	    }
-
-	  case TC_CLASSDESC:
-	    {
-	      ObjectStreamClass osc = readClassDescriptor();
-	      
-	      if (!is_consumed)
-		{
-		  byte b = this.realInputStream.readByte();
-		  if (b != TC_ENDBLOCKDATA)
-		    throw new IOException("Data annotated to class was not consumed." + b);
-		}
-	      else
-		is_consumed = false;
-	      
-	      osc.setSuperclass ((ObjectStreamClass)readObject());
-	      ret_val = osc;
-	      break;
-	    }
-
-	  case TC_STRING:
-	  case TC_LONGSTRING:
-	    {
-	      if(dump) dumpElement("STRING=");
-	      String s = this.realInputStream.readUTF();
-	      if(dump) dumpElementln(s);
-	      ret_val = processResolution(null, s, assignNewHandle(s));
-	      break;
-	    }
-
-	  case TC_ARRAY:
-	    {
-	      if(dump) dumpElementln("ARRAY");
-	      ObjectStreamClass osc = (ObjectStreamClass)readObject();
-	      Class componentType = osc.forClass().getComponentType();
-	      if(dump) dumpElement("ARRAY LENGTH=");
-	      int length = this.realInputStream.readInt();
-	      if(dump) dumpElementln (length + "; COMPONENT TYPE=" + componentType);
-	      Object array = Array.newInstance(componentType, length);
-	      int handle = assignNewHandle(array);
-	      readArrayElements(array, componentType);
-	      if(dump)
-	        for (int i = 0, len = Array.getLength(array); i < len; i++)
-		  dumpElementln("  ELEMENT[" + i + "]=" + Array.get(array, i));
-	      ret_val = processResolution(null, array, handle);
-	      break;
-	    }
-
-	  case TC_OBJECT:
-	    {
-	      if(dump) dumpElementln("OBJECT");
-	      ObjectStreamClass osc = (ObjectStreamClass)readObject();
-	      Class clazz = osc.forClass();
-	      
-	      if (!osc.realClassIsSerializable)
-		throw new NotSerializableException
-		  (clazz + " is not Serializable, and thus cannot be deserialized.");
-	      
-	      if (osc.realClassIsExternalizable)
-		{
-		  Externalizable obj = osc.newInstance();
-		  
-		  int handle = assignNewHandle(obj);
-		  
-		  boolean read_from_blocks = ((osc.getFlags() & SC_BLOCK_DATA) != 0);
-		  
-		  boolean oldmode = this.readDataFromBlock;
-		  if (read_from_blocks)
-		    setBlockDataMode(true);
-		  
-		  obj.readExternal(this);
-		  
-		  if (read_from_blocks)
-                    {
-		      setBlockDataMode(oldmode);
-                      if (!oldmode)
-			if (this.realInputStream.readByte() != TC_ENDBLOCKDATA)
-			    throw new IOException("No end of block data seen for class with readExternal (ObjectInputStream) method.");
-                    }
-		  
-		  ret_val = processResolution(osc, obj, handle);
-		  break;
-		} // end if (osc.realClassIsExternalizable)
-
-	      Object obj = newObject(clazz, osc.firstNonSerializableParentConstructor);
-	      
-	      int handle = assignNewHandle(obj);
-	      Object prevObject = this.currentObject;
-	      ObjectStreamClass prevObjectStreamClass = this.currentObjectStreamClass;
-	      
-	      this.currentObject = obj;
-	      ObjectStreamClass[] hierarchy =
-		inputGetObjectStreamClasses(clazz);
-	      
-	      for (int i = 0; i < hierarchy.length; i++)
-		{
-		  this.currentObjectStreamClass = hierarchy[i];
-		  
-		  if(dump) dumpElementln("Reading fields of " + this.currentObjectStreamClass.getName ());
-
-		  // XXX: should initialize fields in classes in the hierarchy
-		  // that aren't in the stream
-		  // should skip over classes in the stream that aren't in the
-		  // real classes hierarchy
-		  
-		  Method readObjectMethod = this.currentObjectStreamClass.readObjectMethod;
-		  if (readObjectMethod != null)
-		    {
-		      fieldsAlreadyRead = false;
-		      boolean oldmode = setBlockDataMode(true);
-		      callReadMethod(readObjectMethod, this.currentObjectStreamClass.forClass(), obj);
-		      setBlockDataMode(oldmode);
-		    }
-		  else
-		    {
-		      readFields(obj, currentObjectStreamClass);
-		    }
-
-		  if (this.currentObjectStreamClass.hasWriteMethod())
-		    {
-		      if(dump) dumpElement("ENDBLOCKDATA? ");
-		      try
-			{
-			  // FIXME: XXX: This try block is to
-			  // catch EOF which is thrown for some
-			  // objects.  That indicates a bug in
-			  // the logic.
-
-			  if (this.realInputStream.readByte() != TC_ENDBLOCKDATA)
-			    throw new IOException
-			      ("No end of block data seen for class with readObject (ObjectInputStream) method.");
-			  if(dump) dumpElementln("yes");
-			}
-// 		      catch (EOFException e)
-// 			{
-// 			  if(dump) dumpElementln("no, got EOFException");
-// 			}
-		      catch (IOException e)
-			{
-			  if(dump) dumpElementln("no, got IOException");
-			}
-		    }
-		}
-
-	      this.currentObject = prevObject;
-	      this.currentObjectStreamClass = prevObjectStreamClass;
-	      ret_val = processResolution(osc, obj, handle);
-		  
-	      break;
-	    }
-
-	  case TC_RESET:
-	    if(dump) dumpElementln("RESET");
-	    clearHandles();
-	    ret_val = readObject();
-	    break;
-
-	  case TC_EXCEPTION:
-	    {
-	      if(dump) dumpElement("EXCEPTION=");
-	      Exception e = (Exception)readObject();
-	      if(dump) dumpElementln(e.toString());
-	      clearHandles();
-	      throw new WriteAbortedException("Exception thrown during writing of stream", e);
-	    }
-
-	  default:
-	    throw new IOException("Unknown marker on stream: " + marker);
-	  }
+ 	ret_val = parseContent(marker);
       }
     finally
       {
-	setBlockDataMode(old_mode);
-	
-	this.isDeserializing = was_deserializing;
-	
-	depth -= 2;
-	
-	if (! was_deserializing)
-	  {
-	    if (validators.size() > 0)
-	      invokeValidators();
-	  }
+ 	setBlockDataMode(old_mode);
+ 	if (DEBUG)
+	  depth -= 2;
       }
     
     return ret_val;
   }
 
+   /**
+    * Handles a content block within the stream, which begins with a marker
+    * byte indicating its type.
+    *
+    * @param marker the byte marker.
+    * @return an object which represents the parsed content.
+    * @throws ClassNotFoundException if the class of an object being
+    *                                read in cannot be found.
+    * @throws IOException if invalid data occurs or one is thrown by the
+    *                     underlying <code>InputStream</code>.
+    */
+   private Object parseContent(byte marker)
+     throws ClassNotFoundException, IOException
+   {
+     Object ret_val;
+     boolean is_consumed = false;
+
+     switch (marker)
+       {
+       case TC_ENDBLOCKDATA:
+ 	{
+ 	  ret_val = null;
+ 	  is_consumed = true;
+ 	  break;
+ 	}
+ 	
+       case TC_BLOCKDATA:
+       case TC_BLOCKDATALONG:
+ 	{
+ 	  if (marker == TC_BLOCKDATALONG)
+ 	    { if(dump) dumpElementln("BLOCKDATALONG"); }
+ 	  else
+ 	    { if(dump) dumpElementln("BLOCKDATA"); }
+ 	  readNextBlock(marker);
+ 	}
+ 	
+       case TC_NULL:
+ 	{
+ 	  if(dump) dumpElementln("NULL");
+ 	  ret_val = null;
+ 	  break;
+ 	}
+ 	
+       case TC_REFERENCE:
+ 	{
+ 	  if(dump) dumpElement("REFERENCE ");
+ 	  Integer oid = new Integer(this.realInputStream.readInt());
+ 	  if(dump) dumpElementln(Integer.toHexString(oid.intValue()));
+ 	  ret_val = ((ObjectIdentityWrapper)
+ 		     this.objectLookupTable.get(oid)).object;
+ 	  break;
+ 	}
+ 	
+       case TC_CLASS:
+ 	{
+ 	  if(dump) dumpElementln("CLASS");
+ 	  ObjectStreamClass osc = (ObjectStreamClass)readObject();
+ 	  Class clazz = osc.forClass();
+ 	  assignNewHandle(clazz);
+ 	  ret_val = clazz;
+ 	  break;
+ 	}
+ 	
+       case TC_PROXYCLASSDESC:
+ 	{
+ 	  if(dump) dumpElementln("PROXYCLASS");
+ 	  int n_intf = this.realInputStream.readInt();
+ 	  String[] intfs = new String[n_intf];
+ 	  for (int i = 0; i < n_intf; i++)
+ 	    {
+ 	      intfs[i] = this.realInputStream.readUTF();
+ 	    }
+ 	  
+ 	  boolean oldmode = setBlockDataMode(true);
+ 	  Class cl = resolveProxyClass(intfs);
+ 	  setBlockDataMode(oldmode);
+ 	  
+ 	  ObjectStreamClass osc = lookupClass(cl);
+          if (osc.firstNonSerializableParentConstructor == null)
+            {
+              osc.realClassIsSerializable = true;
+              osc.fields = osc.fieldMapping = new ObjectStreamField[0];
+              try
+                {
+                  osc.firstNonSerializableParentConstructor =
+                    Object.class.getConstructor(new Class[0]);
+                }
+              catch (NoSuchMethodException x)
+                {
+                  throw (InternalError)
+                    new InternalError("Object ctor missing").initCause(x);
+                }
+            }
+ 	  assignNewHandle(osc);
+ 	  
+ 	  if (!is_consumed)
+ 	    {
+ 	      byte b = this.realInputStream.readByte();
+ 	      if (b != TC_ENDBLOCKDATA)
+ 		throw new IOException("Data annotated to class was not consumed." + b);
+ 	    }
+ 	  else
+ 	    is_consumed = false;
+ 	  ObjectStreamClass superosc = (ObjectStreamClass)readObject();
+ 	  osc.setSuperclass(superosc);
+ 	  ret_val = osc;
+ 	  break;
+ 	}
+ 	
+       case TC_CLASSDESC:
+ 	{
+ 	  ObjectStreamClass osc = readClassDescriptor();
+ 	  
+ 	  if (!is_consumed)
+ 	    {
+ 	      byte b = this.realInputStream.readByte();
+ 	      if (b != TC_ENDBLOCKDATA)
+ 		throw new IOException("Data annotated to class was not consumed." + b);
+ 	    }
+ 	  else
+ 	    is_consumed = false;
+ 	  
+ 	  osc.setSuperclass ((ObjectStreamClass)readObject());
+ 	  ret_val = osc;
+ 	  break;
+ 	}
+ 	
+       case TC_STRING:
+       case TC_LONGSTRING:
+ 	{
+ 	  if(dump) dumpElement("STRING=");
+ 	  String s = this.realInputStream.readUTF();
+ 	  if(dump) dumpElementln(s);
+ 	  ret_val = processResolution(null, s, assignNewHandle(s));
+ 	  break;
+ 	}
+ 
+       case TC_ARRAY:
+ 	{
+ 	  if(dump) dumpElementln("ARRAY");
+ 	  ObjectStreamClass osc = (ObjectStreamClass)readObject();
+ 	  Class componentType = osc.forClass().getComponentType();
+ 	  if(dump) dumpElement("ARRAY LENGTH=");
+ 	  int length = this.realInputStream.readInt();
+ 	  if(dump) dumpElementln (length + "; COMPONENT TYPE=" + componentType);
+ 	  Object array = Array.newInstance(componentType, length);
+ 	  int handle = assignNewHandle(array);
+ 	  readArrayElements(array, componentType);
+ 	  if(dump)
+ 	    for (int i = 0, len = Array.getLength(array); i < len; i++)
+ 	      dumpElementln("  ELEMENT[" + i + "]=" + Array.get(array, i));
+ 	  ret_val = processResolution(null, array, handle);
+ 	  break;
+ 	}
+ 	
+       case TC_OBJECT:
+ 	{
+ 	  if(dump) dumpElementln("OBJECT");
+ 	  ObjectStreamClass osc = (ObjectStreamClass)readObject();
+ 	  Class clazz = osc.forClass();
+ 	  
+ 	  if (!osc.realClassIsSerializable)
+ 	    throw new NotSerializableException
+ 	      (clazz + " is not Serializable, and thus cannot be deserialized.");
+ 	  
+ 	  if (osc.realClassIsExternalizable)
+	    {
+ 	      Externalizable obj = osc.newInstance();
+	      
+ 	      int handle = assignNewHandle(obj);
+	      
+ 	      boolean read_from_blocks = ((osc.getFlags() & SC_BLOCK_DATA) != 0);
+	      
+ 	      boolean oldmode = this.readDataFromBlock;
+ 	      if (read_from_blocks)
+ 		setBlockDataMode(true);
+	      
+ 	      obj.readExternal(this);
+ 	      
+ 	      if (read_from_blocks)
+                {
+ 		  setBlockDataMode(oldmode);
+ 		  if (!oldmode)
+ 		    if (this.realInputStream.readByte() != TC_ENDBLOCKDATA)
+ 		      throw new IOException("No end of block data seen for class with readExternal (ObjectInputStream) method.");
+		}
+
+ 	      ret_val = processResolution(osc, obj, handle);
+              break;
+	      
+ 	    } // end if (osc.realClassIsExternalizable)
+ 	  
+ 	  Object obj = newObject(clazz, osc.firstNonSerializableParentConstructor);
+ 	  
+ 	  int handle = assignNewHandle(obj);
+ 	  Object prevObject = this.currentObject;
+ 	  ObjectStreamClass prevObjectStreamClass = this.currentObjectStreamClass;
+	  TreeSet prevObjectValidators = this.currentObjectValidators;
+ 	  
+ 	  this.currentObject = obj;
+	  this.currentObjectValidators = null;
+ 	  ObjectStreamClass[] hierarchy =
+ 	    inputGetObjectStreamClasses(clazz);
+ 	  
+ 	  for (int i = 0; i < hierarchy.length; i++)      
+          {
+ 	      this.currentObjectStreamClass = hierarchy[i];
+ 	      if(dump) dumpElementln("Reading fields of " + this.currentObjectStreamClass.getName ());
+  	      
+ 	      // XXX: should initialize fields in classes in the hierarchy
+ 	      // that aren't in the stream
+ 	      // should skip over classes in the stream that aren't in the
+ 	      // real classes hierarchy
+  	      
+ 	      Method readObjectMethod = this.currentObjectStreamClass.readObjectMethod;
+ 	      if (readObjectMethod != null)
+ 		{
+ 		  fieldsAlreadyRead = false;
+ 		  boolean oldmode = setBlockDataMode(true);
+ 		  callReadMethod(readObjectMethod, this.currentObjectStreamClass.forClass(), obj);
+ 		  setBlockDataMode(oldmode);
+ 		}
+ 	      else
+ 		{
+ 		  readFields(obj, currentObjectStreamClass);
+ 		}
+  	      
+ 	      if (this.currentObjectStreamClass.hasWriteMethod())
+  		{
+ 		  if(dump) dumpElement("ENDBLOCKDATA? ");
+ 		  try
+  		    {
+ 		      /* Read blocks until an end marker */
+ 		      byte writeMarker = this.realInputStream.readByte();
+ 		      while (writeMarker != TC_ENDBLOCKDATA)
+			{	
+ 			  parseContent(writeMarker);
+ 			  writeMarker = this.realInputStream.readByte();
+			}
+ 		      if(dump) dumpElementln("yes");
+ 		    }
+ 		  catch (EOFException e)
+ 		    {
+ 		      throw (IOException) new IOException
+ 			("No end of block data seen for class with readObject (ObjectInputStream) method.").initCause(e);
+		    }
+		}
+	    }
+ 	  
+ 	  this.currentObject = prevObject;
+ 	  this.currentObjectStreamClass = prevObjectStreamClass;
+ 	  ret_val = processResolution(osc, obj, handle);
+	  if (currentObjectValidators != null)
+	    invokeValidators();
+	  this.currentObjectValidators = prevObjectValidators;
+
+ 	  break;
+ 	}
+	
+       case TC_RESET:
+ 	if(dump) dumpElementln("RESET");
+ 	clearHandles();
+ 	ret_val = readObject();
+ 	break;
+	
+       case TC_EXCEPTION:
+ 	{
+ 	  if(dump) dumpElement("EXCEPTION=");
+ 	  Exception e = (Exception)readObject();
+ 	  if(dump) dumpElementln(e.toString());
+ 	  clearHandles();
+ 	  throw new WriteAbortedException("Exception thrown during writing of stream", e);
+ 	}
+	
+       default:
+ 	throw new IOException("Unknown marker on stream: " + marker);
+      }
+    return ret_val;
+  }
+
   /**
    * This method makes a partial check of types for the fields
    * contained given in arguments. It checks primitive types of
@@ -501,8 +521,7 @@ public class ObjectInputStream extends InputStream
 						  flags, fields);
     assignNewHandle(osc);
 
-    if (callersClassLoader == null)
-      callersClassLoader = currentLoader();
+    ClassLoader callersClassLoader = currentLoader();
 	      
     for (int i = 0; i < field_count; i++)
       {
@@ -528,36 +547,7 @@ public class ObjectInputStream extends InputStream
 	      
     /* Now that fields have been read we may resolve the class
      * (and read annotation if needed). */
-    Class clazz;
-    try
-      {
-	clazz = resolveClass(osc);
-      }
-    catch (ClassNotFoundException cnfe)
-      {
-	// Maybe it was an primitive class?
-	if (name.equals("void"))
-	  clazz = Void.TYPE;
-	else if (name.equals("boolean"))
-	  clazz = Boolean.TYPE;
-	else if (name.equals("byte"))
-	  clazz = Byte.TYPE;
-	else if (name.equals("short"))
-	  clazz = Short.TYPE;
-	else if (name.equals("char"))
-	  clazz = Character.TYPE;
-	else if (name.equals("int"))
-	  clazz = Integer.TYPE;
-	else if (name.equals("long"))
-	  clazz = Long.TYPE;
-	else if (name.equals("float"))
-	  clazz = Float.TYPE;
-	else if (name.equals("double"))
-	  clazz = Double.TYPE;
-	else
-	  throw cnfe;
-      }
-
+    Class clazz = resolveClass(osc);
     boolean oldmode = setBlockDataMode(true);
     osc.setClass(clazz, lookupClass(clazz.getSuperclass()));
     classLookupTable.put(clazz, osc);
@@ -746,8 +736,10 @@ public class ObjectInputStream extends InputStream
       throw new InvalidObjectException("attempt to add a null "
 				       + "ObjectInputValidation object");
 
-    this.validators.addElement(new ValidatorAndPriority (validator,
-							 priority));
+    if (currentObjectValidators == null)
+      currentObjectValidators = new TreeSet();
+    
+    currentObjectValidators.add(new ValidatorAndPriority(validator, priority));
   }
 
 
@@ -770,33 +762,43 @@ public class ObjectInputStream extends InputStream
   protected Class resolveClass(ObjectStreamClass osc)
     throws ClassNotFoundException, IOException
   {
-    if (callersClassLoader == null)
+    String name = osc.getName();
+    try
       {
-	callersClassLoader = currentLoader ();
-	if (Configuration.DEBUG && dump)
-	  {
-	    dumpElementln ("CallersClassLoader = " + callersClassLoader);
-	  }
+        return Class.forName(name, true, currentLoader());
+      }
+    catch(ClassNotFoundException x)
+      {
+        if (name.equals("void"))
+          return Void.TYPE;
+        else if (name.equals("boolean"))
+          return Boolean.TYPE;
+        else if (name.equals("byte"))
+          return Byte.TYPE;
+        else if (name.equals("char"))
+          return Character.TYPE;
+        else if (name.equals("short"))
+          return Short.TYPE;
+        else if (name.equals("int"))
+          return Integer.TYPE;
+        else if (name.equals("long"))
+          return Long.TYPE;
+        else if (name.equals("float"))
+          return Float.TYPE;
+        else if (name.equals("double"))
+          return Double.TYPE;
+        else
+          throw x;
       }
-
-    return Class.forName(osc.getName(), true, callersClassLoader);
   }
 
   /**
-   * This method invokes the method currentClassLoader for the
-   * current security manager (or build an empty one if it is not
-   * present).
-   *
-   * @return The most recent non-system ClassLoader on the execution stack.
-   * @see java.lang.SecurityManager#currentClassLoader()
+   * Returns the most recent user defined ClassLoader on the execution stack
+   * or null if none is found.
    */
   private ClassLoader currentLoader()
   {
-    SecurityManager sm = System.getSecurityManager();
-    if (sm == null)
-      sm = new SecurityManager () {};
-    
-    ClassLoader cl = currentClassLoader(sm);
+    ClassLoader cl = VMObjectInputStream.currentClassLoader();
     if(cl == null) {
 	cl = ClassLoader.getSystemClassLoader();
     }
@@ -830,7 +832,7 @@ public class ObjectInputStream extends InputStream
 
   /**
    * Reconstruct class hierarchy the same way
-   * {@link java.io.ObjectStreamClass.getObjectStreamClasses(java.lang.Class)} does
+   * {@link java.io.ObjectStreamClass#getObjectStreamClasses(Class)} does
    * but using lookupClass instead of ObjectStreamClass.lookup. This
    * dup is necessary localize the lookup table. Hopefully some future
    * rewritings will be able to prevent this.
@@ -888,16 +890,12 @@ public class ObjectInputStream extends InputStream
   protected Class resolveProxyClass(String[] intfs)
     throws IOException, ClassNotFoundException
   {
-    SecurityManager sm = System.getSecurityManager();
+    ClassLoader cl = currentLoader();
     
-    if (sm == null)
-      sm = new SecurityManager() {};
-    
-    ClassLoader cl = currentClassLoader(sm);
     if(cl == null) {
 	cl = ClassLoader.getSystemClassLoader();
     }
-    
+
     Class[] clss = new Class[intfs.length];
     if(cl == null)
       {
@@ -907,7 +905,7 @@ public class ObjectInputStream extends InputStream
       }
     else
       for (int i = 0; i < intfs.length; i++)
-	clss[i] = cl.loadClass(intfs[i]);
+	clss[i] = Class.forName(intfs[i], false, cl);
     try 
       {
 	return Proxy.getProxyClass(cl, clss);
@@ -978,19 +976,13 @@ public class ObjectInputStream extends InputStream
   {
     if (this.readDataFromBlock)
       {
-	if (this.blockDataPosition + length > this.blockDataBytes)
-	  {
-	    int remain = this.blockDataBytes - this.blockDataPosition;
-	    if (remain != 0)
-	      {
-		System.arraycopy(this.blockData, this.blockDataPosition,
-				 data, offset, remain);
-		offset += remain;
-		length -= remain;
-	      }
-	    readNextBlock ();
-	  }
-
+        int remain = this.blockDataBytes - this.blockDataPosition;
+        if (remain == 0)
+          {
+            readNextBlock();
+            remain = this.blockDataBytes - this.blockDataPosition;
+          }
+        length = Math.min(length, remain);
 	System.arraycopy(this.blockData, this.blockDataPosition,
 			 data, offset, length);
 	this.blockDataPosition += length;
@@ -1234,7 +1226,7 @@ public class ObjectInputStream extends InputStream
    * This method should be called by a method called 'readObject' in the
    * deserializing class (if present). It cannot (and should not)be called
    * outside of it. Its goal is to read all fields in the real input stream
-   * and keep them accessible through the {@link #GetField} class. Calling
+   * and keep them accessible through the {@link GetField} class. Calling
    * this method will not alter the deserializing object.
    *
    * @return A valid freshly created 'GetField' instance to get access to
@@ -1302,7 +1294,7 @@ public class ObjectInputStream extends InputStream
 	    }
 	  catch (NoSuchFieldException e)
 	    {
-	      throw new IllegalArgumentException(e.getMessage());
+	      throw new IllegalArgumentException(e);
 	    }
 	}
 
@@ -1447,6 +1439,7 @@ public class ObjectInputStream extends InputStream
 	  ObjectStreamField field = clazz.getField(name);
 	  boolean illegal = false;
 
+          // XXX This code is horrible and needs to be rewritten!
 	  try
 	    {
 	      try
@@ -1506,7 +1499,7 @@ public class ObjectInputStream extends InputStream
 	      catch (NoSuchFieldException e)
 		{
 		  if (field == null)
-		    throw new IllegalArgumentException(e.getMessage());
+		    throw new IllegalArgumentException(e);
 		}
 	       
 	    }
@@ -1846,12 +1839,12 @@ public class ObjectInputStream extends InputStream
         throw new InvalidClassException("Missing accessible no-arg base class constructor for " + real_class.getName()); 
     try
       {
-	return allocateObject(real_class, constructor.getDeclaringClass(), constructor);
+	return VMObjectInputStream.allocateObject(real_class, constructor.getDeclaringClass(), constructor);
       }
     catch (InstantiationException e)
       {
-        throw new ClassNotFoundException
-		("Instance of " + real_class + " could not be created");
+        throw (ClassNotFoundException) new ClassNotFoundException
+          ("Instance of " + real_class + " could not be created").initCause(e);
       }
   }
 
@@ -1859,30 +1852,22 @@ public class ObjectInputStream extends InputStream
   // on OBJ
   private void invokeValidators() throws InvalidObjectException
   {
-    Object[] validators = new Object[this.validators.size()];
-    this.validators.copyInto (validators);
-    Arrays.sort (validators);
-
     try
       {
-	for (int i=0; i < validators.length; i++)
-	  ((ObjectInputValidation)validators[i]).validateObject();
+	Iterator it = currentObjectValidators.iterator();
+	while(it.hasNext())
+	  {
+	    ValidatorAndPriority vap = (ValidatorAndPriority) it.next();
+	    ObjectInputValidation validator = vap.validator;
+	    validator.validateObject();
+	  }
       }
     finally
       {
-	this.validators.removeAllElements();
+	currentObjectValidators = null;
       }
   }
 
-  /**
-   * This native method is used to get access to the protected method
-   * of the same name in SecurityManger.
-   *
-   * @param sm SecurityManager instance which should be called.
-   * @return The current class loader in the calling stack.
-   */
-  private static native ClassLoader currentClassLoader (SecurityManager sm);
-
   private void callReadMethod (Method readObject, Class klass, Object obj)
     throws ClassNotFoundException, IOException
   {
@@ -1901,22 +1886,19 @@ public class ObjectInputStream extends InputStream
         if (exception instanceof ClassNotFoundException)
           throw (ClassNotFoundException) exception;
 
-	throw new IOException("Exception thrown from readObject() on " +
-			       klass + ": " + exception.getClass().getName());
+	throw (IOException) new IOException(
+	  "Exception thrown from readObject() on " + klass).initCause(x);
       }
     catch (Exception x)
       {
-	throw new IOException("Failure invoking readObject() on " +
-			       klass + ": " + x.getClass().getName());
+	throw (IOException) new IOException(
+	  "Failure invoking readObject() on " + klass).initCause(x);
       }
 
     // Invalidate fields which has been read through readFields.
     prereadFields = null;
   }
     
-  private native Object allocateObject(Class clazz, Class constr_clazz, Constructor constructor)
-    throws InstantiationException;
-
   private static final int BUFFER_SIZE = 1024;
 
   private DataInputStream realInputStream;
@@ -1931,19 +1913,19 @@ public class ObjectInputStream extends InputStream
   private Hashtable objectLookupTable;
   private Object currentObject;
   private ObjectStreamClass currentObjectStreamClass;
+  private TreeSet currentObjectValidators;
   private boolean readDataFromBlock;
-  private boolean isDeserializing;
   private boolean fieldsAlreadyRead;
-  private Vector validators;
   private Hashtable classLookupTable;
   private GetField prereadFields;
 
-  private ClassLoader callersClassLoader;
   private static boolean dump;
 
   // The nesting depth for debugging output
   private int depth = 0;
 
+  private static final boolean DEBUG = false;
+
   private void dumpElement (String msg)
   {
     System.out.print(msg);
@@ -1957,14 +1939,6 @@ public class ObjectInputStream extends InputStream
     System.out.print (Thread.currentThread() + ": ");
   }
 
-  static
-  {
-    if (Configuration.INIT_LOAD_LIBRARY)
-      {
-	System.loadLibrary ("javaio");
-      }
-  }
-
   // used to keep a prioritized list of object validators
   private static final class ValidatorAndPriority implements Comparable
   {
