@@ -21,8 +21,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -46,7 +46,9 @@ package java.lang;
 import gnu.classpath.VMStackWalker;
 
 import java.io.InputStream;
+import java.io.ObjectStreamClass;
 import java.io.Serializable;
+import java.lang.reflect.Array;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.Field;
 import java.lang.reflect.InvocationTargetException;
@@ -63,7 +65,7 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashMap;
 import java.util.HashSet;
-import gnu.classpath.RawData;
+import gnu.classpath.Pointer;
 
 
 /**
@@ -183,7 +185,7 @@ public final class Class implements Serializable
   /** The class signers. */
   private Object[] signers;
   /** The class protection domain. */
-  private final ProtectionDomain pd;
+  private final transient ProtectionDomain pd;
 
   /* We use an inner class, so that Class doesn't have a static initializer */
   private static final class StaticData
@@ -252,10 +254,10 @@ public final class Class implements Serializable
    */
   public static Class forName(String name) throws ClassNotFoundException
   {
+//    return VMClass.forName(name, true, VMStackWalker.getCallingClassLoader());
     Class result = VMClass.forName (name);
     if (result == null)
-      result = Class.forName(name, true,
-	VMStackWalker.getCallingClassLoader());
+      result = Class.forName(name, true, VMStackWalker.getCallingClassLoader());
     return result;
   }
 
@@ -366,24 +368,8 @@ public final class Class implements Serializable
             if (cl != null)
               sm.checkPermission(new RuntimePermission("getClassLoader"));
           }
-	if (name.startsWith("["))
-	  return VMClass.loadArrayClass(name, null);
-	Class c = VMClassLoader.loadClass(name, true);
-	if (c != null)
-	  {
-	    if (initialize)
-	      VMClass.initialize(c);
-	    return c;
-	  }
-        throw new ClassNotFoundException(name);
       }
-    if (name.startsWith("["))
-      return VMClass.loadArrayClass(name, classloader);
-    Class c = classloader.loadClass(name);
-    classloader.resolveClass(c);
-    if (initialize)
-      VMClass.initialize(c);
-    return c;
+    return VMClass.forName(name, initialize, classloader);
     */
   }
   
@@ -866,7 +852,8 @@ public final class Class implements Serializable
     ClassLoader cl = getClassLoader();
     if (cl != null)
       return cl.getPackage(getPackagePortion(getName()));
-    return null;
+    else
+      return VMClassLoader.getPackage(getPackagePortion(getName()));
   }
 
   /**
@@ -1008,7 +995,7 @@ public final class Class implements Serializable
    * @param list List of methods to search
    * @param name Name of method
    * @param args Method parameter types
-   * @see #getMethod()
+   * @see #getMethod(String, Class[])
    */
   private static Method matchMethod(Method[] list, String name, Class[] args)
   {
@@ -1116,14 +1103,14 @@ public final class Class implements Serializable
    * public and final, but not an interface.
    *
    * @return the modifiers of this class
-   * @see Modifer
+   * @see Modifier
    * @since 1.1
    */
   public int getModifiers()
   {
     // TODO: Check for primitive/void type and possibly array type.
     //       Native method cannot handle them.
-    return VMClass.getModifiers (this);
+    return VMClass.getModifiers (this, false);
   }
 
 
@@ -1251,18 +1238,18 @@ public final class Class implements Serializable
   private String resourcePath(String resourceName)
   {
     if (resourceName.length() > 0)
-    {
-      if (resourceName.charAt(0) != '/')
       {
-        String pkg = getPackagePortion(getName());
-        if (pkg.length() > 0)
-          resourceName = pkg.replace('.','/') + '/' + resourceName;
+	if (resourceName.charAt(0) != '/')
+	  {
+	    String pkg = getPackagePortion(getName());
+	    if (pkg.length() > 0)
+	      resourceName = pkg.replace('.','/') + '/' + resourceName;
+	  }
+	else
+	  {
+	    resourceName = resourceName.substring(1);
+	  }
       }
-      else
-      {
-        resourceName = resourceName.substring(1);
-      }
-    }
     return resourceName;
   }
 
@@ -1468,7 +1455,7 @@ public final class Class implements Serializable
 	if (constructor == null)
 	  throw new InstantiationException(getName());
 	if (!Modifier.isPublic(constructor.getModifiers())
-            || !Modifier.isPublic(getModifiers()))
+            || !Modifier.isPublic(VMClass.getModifiers(this, true)))
 	  {
 	    final Constructor finalConstructor = constructor;
 	    AccessController.doPrivileged(new PrivilegedAction()
@@ -1488,7 +1475,7 @@ public final class Class implements Serializable
       }
     int modifiers = constructor.getModifiers();
     if (!Modifier.isPublic(modifiers)
-        || !Modifier.isPublic(getModifiers()))
+        || !Modifier.isPublic(VMClass.getModifiers(this, true)))
       {
 	Class caller = VMStackWalker.getCallingClass();
 	if (caller != null &&
