@@ -19,8 +19,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -52,6 +52,7 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.net.URL;
 import java.net.URLClassLoader;
+import java.nio.ByteBuffer;
 import java.security.CodeSource;
 import java.security.PermissionCollection;
 import java.security.Policy;
@@ -251,6 +252,11 @@ public abstract class ClassLoader
   // Package visible for use by Class.
   Map classAssertionStatus;
 
+  /**
+   * VM private data.
+   */
+  transient Object vmdata;
+
   /**
    * Create a new ClassLoader with as parent the system classloader. There
    * may be a security check for <code>checkCreateClassLoader</code>.
@@ -327,27 +333,19 @@ public abstract class ClassLoader
   protected Class loadClass(String name, boolean resolve)
     throws ClassNotFoundException
   {
-
-    synchronized(this)
+    synchronized(this) {
+    // Have we already loaded this class?
+    Class c = findLoadedClass(name);
+    if (c == null)
       {
-	if (name.charAt(0) == '[')
-	  {
-	    throw new ClassNotFoundException (name);
-	  }
-	
-	// Have we already loaded this class?
-	Class c = findLoadedClass(name);
-	if (c != null)
-	  {
-	    return c;
-	  }
-	
 	// Can the class be loaded by a parent?
 	try
 	  {
 	    if (parent == null)
 	      {
 		c = VMClassLoader.loadClass(name, resolve);
+		if (c != null)
+		  return c;
 	      }
 	    else
 	      {
@@ -356,16 +354,16 @@ public abstract class ClassLoader
 	  }
 	catch (ClassNotFoundException e)
 	  {
-	    // Still not found, we have to do it ourself.
-	    c = findClass(name);
-	    if (resolve)
-	      {
-		resolveClass(c);
-	      }
 	  }
-	loadedClasses.put(c.getName(), c);
-	return c;
+	// Still not found, we have to do it ourself.
+	c = findClass(name);
+	if (c != null && resolve)
+	    resolveClass(c);
       }
+    else if (resolve)
+      resolveClass(c);
+    return c;
+    }
   }
 
   final Class createArray(String name, boolean resolve)
@@ -382,6 +380,8 @@ public abstract class ClassLoader
 	Class c = findLoadedClass(name);
 	if (c != null)
 	  {
+	    if (resolve)
+	      resolveClass(c);
 	    return c;
 	  }
 	
@@ -493,6 +493,7 @@ public abstract class ClassLoader
 
   final Class defineArray(String name, Class element)
   {
+    synchronized(this) {
     Class retval;
     
     if (name == null)
@@ -523,6 +524,7 @@ public abstract class ClassLoader
       }
 
     return retval;
+    }
   }
  
 
@@ -667,6 +669,10 @@ public abstract class ClassLoader
     if (domain == null)
       domain = StaticData.defaultProtectionDomain;
 
+// FIXME (SableVM)
+//    if (! initialized)
+//      throw new SecurityException("attempt to define class from uninitialized class loader");
+
     partiallyLoadedClasses.add(name);
     try
       {
@@ -687,6 +693,38 @@ public abstract class ClassLoader
 
   /**
    * Links the class, if that has not already been done.
+   * Helper to define a class using the contents of a byte buffer. If
+   * the domain is null, the default of
+   * <code>Policy.getPolicy().getPermissions(new CodeSource(null,
+   * null))</code> is used. Once a class has been defined in a
+   * package, all further classes in that package must have the same
+   * set of certificates or a SecurityException is thrown.
+   *
+   * @param name the name to give the class.  null if unknown
+   * @param buf a byte buffer containing bytes that form a class.
+   * @param domain the ProtectionDomain to give to the class, null for the
+   *        default protection domain
+   * @return the class that was defined
+   * @throws ClassFormatError if data is not in proper classfile format
+   * @throws NoClassDefFoundError if the supplied name is not the same as
+   *                              the one specified by the byte buffer.
+   * @throws SecurityException if name starts with "java.", or if certificates
+   *         do not match up
+   * @since 1.5
+   */
+  protected final Class defineClass(String name, ByteBuffer buf,
+				    ProtectionDomain domain)
+    throws ClassFormatError
+  {
+    byte[] data = new byte[buf.remaining()];
+    buf.get(data);
+    return defineClass(name, data, 0, data.length, domain);
+  }
+
+  /**
+   * Links the class, if that has not already been done. Linking basically
+   * resolves all references to other classes made by this class.
+>>>>>>> .merge-right.r4664
    *
    * @param c the class to resolve
    * @throws NullPointerException if c is null
@@ -709,6 +747,7 @@ public abstract class ClassLoader
   protected final Class findSystemClass(String name)
     throws ClassNotFoundException
   {
+    checkInitialized();
     return Class.forName(name, false, StaticData.systemClassLoader);
   }
 
@@ -745,6 +784,7 @@ public abstract class ClassLoader
    */
   protected final void setSigners(Class c, Object[] signers)
   {
+    checkInitialized();
     c.setSigners(signers);
   }
 
@@ -757,9 +797,9 @@ public abstract class ClassLoader
    */
   protected final Class findLoadedClass(String name)
   {
-    // NOTE: If the VM is keeping its own cache, it may make sense to have
-    // this method be native.
+    checkInitialized();
     return (Class) loadedClasses.get(name);
+//    return VMClassLoader.findLoadedClass(this, name);
   }
 
   /**
@@ -1096,7 +1136,7 @@ public abstract class ClassLoader
    *
    * @param name the (system specific) name of the requested library
    * @return the full pathname to the requested library, or null
-   * @see Runtime#loadLibrary()
+   * @see Runtime#loadLibrary(String)
    * @since 1.2
    */
   protected String findLibrary(String name)
@@ -1126,7 +1166,7 @@ public abstract class ClassLoader
    *
    * @param name the package (and subpackages) to affect
    * @param enabled true to set the default to enabled
-   * @see #setDefaultAssertionStatus(String, boolean)
+   * @see #setDefaultAssertionStatus(boolean)
    * @see #setClassAssertionStatus(String, boolean)
    * @see #clearAssertionStatus()
    * @since 1.4
@@ -1147,7 +1187,7 @@ public abstract class ClassLoader
    * @param name the class to affect
    * @param enabled true to set the default to enabled
    * @throws NullPointerException if name is null
-   * @see #setDefaultAssertionStatus(String, boolean)
+   * @see #setDefaultAssertionStatus(boolean)
    * @see #setPackageAssertionStatus(String, boolean)
    * @see #clearAssertionStatus()
    * @since 1.4
@@ -1261,15 +1301,23 @@ public abstract class ClassLoader
 
   static ClassLoader defaultGetSystemClassLoader()
   {
-    URL[] extURLs = getExtClassLoaderUrls();
-    ClassLoader extClassLoader;
-    if (extURLs.length > 0)
-      extClassLoader = new URLClassLoader(getExtClassLoaderUrls(), null);
-    else
-      extClassLoader = null;
+    return createAuxiliarySystemClassLoader(
+        createSystemClassLoader(getSystemClassLoaderUrls(),
+            createExtClassLoader(getExtClassLoaderUrls(), null)));
+  }
 
-    ClassLoader systemClassLoader =
-	new URLClassLoader(getSystemClassLoaderUrls(), extClassLoader)
+  static ClassLoader createExtClassLoader(URL[] urls, ClassLoader parent)
+  {
+    if (urls.length > 0)
+      return new URLClassLoader(urls, parent);
+    else
+      return parent;
+  }
+
+  static ClassLoader createSystemClassLoader(URL[] urls, ClassLoader parent)
+  {
+    return
+	new URLClassLoader(urls, parent)
 	{
 	    protected synchronized Class loadClass(String name,
 		boolean resolve)
@@ -1285,16 +1333,20 @@ public abstract class ClassLoader
 		return super.loadClass(name, resolve);
 	    }
 	};
+  }
+
+  static ClassLoader createAuxiliarySystemClassLoader(ClassLoader parent)
+  {
     String loader = SystemProperties.getProperty("java.system.class.loader", null);
     if (loader == null)
       {
-	return systemClassLoader;
+	return parent;
       }
     try
       {
-	Constructor c = Class.forName(loader, false, systemClassLoader)
+	Constructor c = Class.forName(loader, false, parent)
 	    .getConstructor(new Class[] { ClassLoader.class });
-	return (ClassLoader)c.newInstance(new Object[] { systemClassLoader });
+	return (ClassLoader)c.newInstance(new Object[] { parent });
       }
     catch (Exception e)
       {
@@ -1304,4 +1356,17 @@ public abstract class ClassLoader
 		.initCause(e);
       }
   }
+
+  /**
+   * Before doing anything "dangerous" please call this method to make sure
+   * this class loader instance was properly constructed (and not obtained
+   * by exploiting the finalizer attack)
+   * @see #initialized
+   */
+  private void checkInitialized()
+  {
+  // FIXME: SableVM
+//    if (! initialized)
+//      throw new SecurityException("attempt to use uninitialized class loader");
+  }
 }
