@@ -19,8 +19,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -699,7 +699,7 @@ public class Runtime
     SecurityManager sm = SecurityManager.current; // Be thread-safe!
     if (sm != null)
       sm.checkRead(filename);
-    return VMRuntime.nativeLoad(filename, loader);
+    return VMRuntime.nativeLoad(filename, (loader == null) ? null : loader.vmData);
   }
 
   /**
@@ -763,17 +763,6 @@ public class Runtime
 	for (int i = 0; i < libpath.length; i++)
 	  if (loadLib(libpath[i] + filename, loader) != 0)
 	    return;
-      /* finally try the boot library path */
-	String path = SystemProperties.getProperty("sablevm.boot.library.path");
-	if (!path.endsWith ("/"))
-	  {
-	    path += "/";
-	  }
-	try {
-	  load (path + filename);
-	  return;
-	  /* we catch this exception to provide a nicer error message */
-	} catch (UnsatisfiedLinkError e) {}
       }
     throw new UnsatisfiedLinkError("Native library `" + libname
       + "' not found (as file `" + filename + "') in gnu.classpath.boot.library.path and java.library.path");
