@@ -20,8 +20,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -42,6 +42,7 @@ exception statement from your version. */
 
 package java.lang;
 
+import java.security.Permission;
 import java.util.Map;
 import java.util.WeakHashMap;
 
@@ -740,7 +741,7 @@ public class Thread implements Runnable
    *
    * @return the context class loader
    * @throws SecurityException when permission is denied
-   * @see setContextClassLoader(ClassLoader)
+   * @see #setContextClassLoader(ClassLoader)
    * @since 1.2
    */
   public synchronized ClassLoader getContextClassLoader()
@@ -750,6 +751,8 @@ public class Thread implements Runnable
     if (sm != null)
       // XXX Don't check this if the caller's class loader is an ancestor.
       sm.checkPermission(new RuntimePermission("getClassLoader"));
+    if (contextClassLoader == null)
+      contextClassLoader = ClassLoader.getSystemClassLoader();
     return contextClassLoader;
   }
 
@@ -762,7 +765,7 @@ public class Thread implements Runnable
    *
    * @param classloader the new context class loader
    * @throws SecurityException when permission is denied
-   * @see getContextClassLoader()
+   * @see #getContextClassLoader()
    * @since 1.2
    */
   public synchronized void setContextClassLoader(ClassLoader classloader)
@@ -848,8 +851,11 @@ public class Thread implements Runnable
   {
 
     // Check parameters
-    if (ms < 0 || ns < 0 || ns > 999999)
-      throw new IllegalArgumentException();
+    if (ms < 0 )
+      throw new IllegalArgumentException("Negative milliseconds: " + ms);
+
+    if (ns < 0 || ns > 999999)
+      throw new IllegalArgumentException("Nanoseconds ouf of range: " + ns);
 
     // Really sleep
     VMThread.sleep(ms, ns);
