@@ -17,8 +17,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -40,9 +40,7 @@ exception statement from your version. */
 
 package java.nio;
 
-import gnu.classpath.RawData;
-import gnu.classpath.RawData32;
-import gnu.classpath.RawData64;
+import gnu.classpath.*;
 
 /**
  * @since 1.4
@@ -57,7 +55,7 @@ public abstract class ByteBuffer extends Buffer
       if(capacity != (int) capacity)
         throw new InternalError("The JNI specification is not coherent.");
 
-      return new DirectByteBufferImpl.ReadWrite(null, new RawData32(address), (int) capacity, (int) capacity, 0);
+      return new DirectByteBufferImpl.ReadWrite(null, new Pointer32(address), (int) capacity, (int) capacity, 0);
     }
 
     static ByteBuffer vmNewDirectByteBuffer64(long address, long capacity)
@@ -65,17 +63,17 @@ public abstract class ByteBuffer extends Buffer
       if(capacity != (int) capacity)
         throw new InternalError("The JNI specification is not coherent.");
 
-      return new DirectByteBufferImpl.ReadWrite(null, new RawData64(address), (int) capacity, (int) capacity, 0);
+      return new DirectByteBufferImpl.ReadWrite(null, new Pointer64(address), (int) capacity, (int) capacity, 0);
     }
 
     static int vmGetDirectBufferAddress32(ByteBuffer buffer)
     {
-      return ((RawData32) ((DirectByteBufferImpl) buffer).address).data;
+      return ((Pointer32) ((DirectByteBufferImpl) buffer).address).data;
     }
   
     static long vmGetDirectBufferAddress64(ByteBuffer buffer)
     {
-      return ((RawData64) ((DirectByteBufferImpl) buffer).address).data;
+      return ((Pointer64) ((DirectByteBufferImpl) buffer).address).data;
     }
   
     static long vmGetDirectBufferCapacity(ByteBuffer buffer)
