@@ -1,5 +1,5 @@
 /* java.util.TimeZone
-   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004
+   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005
    Free Software Foundation, Inc.
 
 Modifications Copyright (C) 2004 by Etienne Gagnon.
@@ -20,8 +20,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -206,6 +206,7 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
 	timezones0.put("MST", tz);
 	timezones0.put("MST7MDT", tz);
 	timezones0.put("America/Boise", tz);
+	timezones0.put("America/Cambridge_Bay", tz);
 	timezones0.put("America/Chihuahua", tz);
 	timezones0.put("America/Denver", tz);
 	timezones0.put("America/Edmonton", tz);
@@ -225,15 +226,15 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
 	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 2000 * 3600);
 	timezones0.put("CST", tz);
 	timezones0.put("CST6CDT", tz);
-	timezones0.put("America/Cambridge_Bay", tz);
 	timezones0.put("America/Cancun", tz);
 	timezones0.put("America/Chicago", tz);
 	timezones0.put("America/Menominee", tz);
 	timezones0.put("America/Merida", tz);
 	timezones0.put("America/Mexico_City", tz);
 	timezones0.put("America/Monterrey", tz);
+	timezones0.put("America/North_Dakota/Center", tz);
 	timezones0.put("America/Rainy_River", tz);
-	timezones0.put("America/Winnipeg", tz);
+	timezones0.put("America/Rankin_Inlet", tz);
 	tz = new SimpleTimeZone(-6000 * 3600, "America/Belize");
 	timezones0.put("America/Belize", tz);
 	timezones0.put("America/Costa_Rica", tz);
@@ -244,16 +245,25 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
 	timezones0.put("America/Swift_Current", tz);
 	timezones0.put("America/Tegucigalpa", tz);
 	timezones0.put("Pacific/Galapagos", tz);
+	tz = new SimpleTimeZone
+	  (-6000 * 3600, "America/Winnipeg",
+	   Calendar.APRIL, 1, Calendar.SUNDAY, 2000 * 3600,
+	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 3000 * 3600);
+	timezones0.put("America/Winnipeg", tz);
 	tz = new SimpleTimeZone
 	  (-6000 * 3600, "Pacific/Easter",
-	   Calendar.OCTOBER, 9, -Calendar.SUNDAY, 0 * 3600,
-	   Calendar.MARCH, 9, -Calendar.SUNDAY, 0 * 3600);
+	   Calendar.OCTOBER, 2, Calendar.SATURDAY, 23000 * 3600,
+	   Calendar.MARCH, 2, Calendar.SATURDAY, 22000 * 3600);
 	timezones0.put("Pacific/Easter", tz);
 	tz = new SimpleTimeZone
 	  (-5000 * 3600, "America/Grand_Turk",
 	   Calendar.APRIL, 1, Calendar.SUNDAY, 0 * 3600,
 	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 0 * 3600);
 	timezones0.put("America/Grand_Turk", tz);
+	tz = new SimpleTimeZone
+	  (-5000 * 3600, "America/Havana",
+	   Calendar.APRIL, 1, Calendar.SUNDAY, 1000 * 3600,
+	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 1000 * 3600);
 	timezones0.put("America/Havana", tz);
 	tz = new SimpleTimeZone(-5000 * 3600, "EST5");
 	timezones0.put("EST5", tz);
@@ -267,14 +277,11 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
 	timezones0.put("America/Indiana/Marengo", tz);
 	timezones0.put("America/Indiana/Vevay", tz);
 	timezones0.put("America/Indianapolis", tz);
-	timezones0.put("America/Iqaluit", tz);
 	timezones0.put("America/Jamaica", tz);
 	timezones0.put("America/Lima", tz);
 	timezones0.put("America/Panama", tz);
-	timezones0.put("America/Pangnirtung", tz);
 	timezones0.put("America/Port-au-Prince", tz);
-	timezones0.put("America/Porto_Acre", tz);
-	timezones0.put("America/Rankin_Inlet", tz);
+	timezones0.put("America/Rio_Branco", tz);
 	tz = new SimpleTimeZone
 	  (-5000 * 3600, "EST",
 	   Calendar.APRIL, 1, Calendar.SUNDAY, 2000 * 3600,
@@ -283,6 +290,7 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
 	timezones0.put("EST5EDT", tz);
 	timezones0.put("EDT", tz);
 	timezones0.put("America/Detroit", tz);
+	timezones0.put("America/Iqaluit", tz);
 	timezones0.put("America/Kentucky/Louisville", tz);
 	timezones0.put("America/Kentucky/Monticello", tz);
 	timezones0.put("America/Louisville", tz);
@@ -290,7 +298,9 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
 	timezones0.put("America/Nassau", tz);
 	timezones0.put("America/New_York", tz);
 	timezones0.put("America/Nipigon", tz);
+	timezones0.put("America/Pangnirtung", tz);
 	timezones0.put("America/Thunder_Bay", tz);
+	timezones0.put("America/Toronto", tz);
 	tz = new SimpleTimeZone(-4000 * 3600, "PRT");
 	timezones0.put("PRT", tz);
 	timezones0.put("America/Anguilla", tz);
@@ -319,19 +329,25 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
 	timezones0.put("America/Tortola", tz);
 	tz = new SimpleTimeZone
 	  (-4000 * 3600, "America/Asuncion",
-	   Calendar.OCTOBER, 1, Calendar.SUNDAY, 0 * 3600,
-	   Calendar.FEBRUARY, -1, Calendar.SUNDAY, 0 * 3600);
+	   Calendar.OCTOBER, 3, Calendar.SUNDAY, 0 * 3600,
+	   Calendar.MARCH, 2, Calendar.SUNDAY, 0 * 3600);
 	timezones0.put("America/Asuncion", tz);
 	tz = new SimpleTimeZone
-	  (-4000 * 3600, "America/Cuiaba",
-	   Calendar.OCTOBER, 2, Calendar.SUNDAY, 0 * 3600,
+	  (-4000 * 3600, "America/Campo_Grande",
+	   Calendar.OCTOBER, 3, Calendar.SUNDAY, 0 * 3600,
 	   Calendar.FEBRUARY, 3, Calendar.SUNDAY, 0 * 3600);
+	timezones0.put("America/Campo_Grande", tz);
 	timezones0.put("America/Cuiaba", tz);
 	tz = new SimpleTimeZone
 	  (-4000 * 3600, "America/Goose_Bay",
 	   Calendar.APRIL, 1, Calendar.SUNDAY, 60000,
 	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 60000);
 	timezones0.put("America/Goose_Bay", tz);
+	tz = new SimpleTimeZone
+	  (-4000 * 3600, "America/Santiago",
+	   Calendar.OCTOBER, 9, -Calendar.SUNDAY, 1000 * 3600,
+	   Calendar.MARCH, 9, -Calendar.SUNDAY, 0 * 3600);
+	timezones0.put("America/Santiago", tz);
 	tz = new SimpleTimeZone
 	  (-4000 * 3600, "America/Glace_Bay",
 	   Calendar.APRIL, 1, Calendar.SUNDAY, 2000 * 3600,
@@ -341,15 +357,14 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
 	timezones0.put("America/Thule", tz);
 	timezones0.put("Atlantic/Bermuda", tz);
 	tz = new SimpleTimeZone
-	  (-4000 * 3600, "America/Santiago",
+	  (-4000 * 3600, "Antarctica/Palmer",
 	   Calendar.OCTOBER, 9, -Calendar.SUNDAY, 0 * 3600,
 	   Calendar.MARCH, 9, -Calendar.SUNDAY, 0 * 3600);
-	timezones0.put("America/Santiago", tz);
 	timezones0.put("Antarctica/Palmer", tz);
 	tz = new SimpleTimeZone
 	  (-4000 * 3600, "Atlantic/Stanley",
-	   Calendar.SEPTEMBER, 2, Calendar.SUNDAY, 0 * 3600,
-	   Calendar.APRIL, 16, -Calendar.SUNDAY, 0 * 3600);
+	   Calendar.SEPTEMBER, 1, Calendar.SUNDAY, 2000 * 3600,
+	   Calendar.APRIL, 3, Calendar.SUNDAY, 2000 * 3600);
 	timezones0.put("Atlantic/Stanley", tz);
 	tz = new SimpleTimeZone
 	  (-3500 * 3600, "CNT",
@@ -357,49 +372,55 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
 	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 60000);
 	timezones0.put("CNT", tz);
 	timezones0.put("America/St_Johns", tz);
-	tz = new SimpleTimeZone
-	  (-3000 * 3600, "America/Araguaina",
-	   Calendar.OCTOBER, 2, Calendar.SUNDAY, 0 * 3600,
-	   Calendar.FEBRUARY, 3, Calendar.SUNDAY, 0 * 3600);
-	timezones0.put("America/Araguaina", tz);
-	timezones0.put("America/Sao_Paulo", tz);
-	tz = new SimpleTimeZone(-3000 * 3600, "AGT");
-	timezones0.put("AGT", tz);
-	timezones0.put("America/Belem", tz);
-	timezones0.put("America/Buenos_Aires", tz);
-	timezones0.put("America/Catamarca", tz);
-	timezones0.put("America/Cayenne", tz);
-	timezones0.put("America/Cordoba", tz);
-	timezones0.put("America/Fortaleza", tz);
-	timezones0.put("America/Jujuy", tz);
-	timezones0.put("America/Maceio", tz);
-	timezones0.put("America/Mendoza", tz);
-	timezones0.put("America/Montevideo", tz);
-	timezones0.put("America/Paramaribo", tz);
-	timezones0.put("America/Recife", tz);
-	timezones0.put("America/Rosario", tz);
 	tz = new SimpleTimeZone
 	  (-3000 * 3600, "America/Godthab",
-	   Calendar.MARCH, 30, -Calendar.SATURDAY, 22000 * 3600,
-	   Calendar.OCTOBER, 30, -Calendar.SATURDAY, 22000 * 3600);
+	   Calendar.MARCH, 30, -Calendar.SATURDAY, 23000 * 3600,
+	   Calendar.OCTOBER, 30, -Calendar.SATURDAY, 23000 * 3600);
 	timezones0.put("America/Godthab", tz);
 	tz = new SimpleTimeZone
 	  (-3000 * 3600, "America/Miquelon",
 	   Calendar.APRIL, 1, Calendar.SUNDAY, 2000 * 3600,
 	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 2000 * 3600);
 	timezones0.put("America/Miquelon", tz);
+	tz = new SimpleTimeZone
+	  (-3000 * 3600, "America/Sao_Paulo",
+	   Calendar.OCTOBER, 3, Calendar.SUNDAY, 0 * 3600,
+	   Calendar.FEBRUARY, 3, Calendar.SUNDAY, 0 * 3600);
+	timezones0.put("America/Sao_Paulo", tz);
+	tz = new SimpleTimeZone(-3000 * 3600, "AGT");
+	timezones0.put("AGT", tz);
+	timezones0.put("America/Araguaina", tz);
+	timezones0.put("America/Argentina/Buenos_Aires", tz);
+	timezones0.put("America/Argentina/Catamarca", tz);
+	timezones0.put("America/Argentina/ComodRivadavia", tz);
+	timezones0.put("America/Argentina/Cordoba", tz);
+	timezones0.put("America/Argentina/Jujuy", tz);
+	timezones0.put("America/Argentina/La_Rioja", tz);
+	timezones0.put("America/Argentina/Mendoza", tz);
+	timezones0.put("America/Argentina/Rio_Gallegos", tz);
+	timezones0.put("America/Argentina/San_Juan", tz);
+	timezones0.put("America/Argentina/Tucuman", tz);
+	timezones0.put("America/Argentina/Ushuaia", tz);
+	timezones0.put("America/Bahia", tz);
+	timezones0.put("America/Belem", tz);
+	timezones0.put("America/Cayenne", tz);
+	timezones0.put("America/Fortaleza", tz);
+	timezones0.put("America/Maceio", tz);
+	timezones0.put("America/Montevideo", tz);
+	timezones0.put("America/Paramaribo", tz);
+	timezones0.put("America/Recife", tz);
+	timezones0.put("Antarctica/Rothera", tz);
 	tz = new SimpleTimeZone(-2000 * 3600, "America/Noronha");
 	timezones0.put("America/Noronha", tz);
 	timezones0.put("Atlantic/South_Georgia", tz);
 	tz = new SimpleTimeZone
 	  (-1000 * 3600, "America/Scoresbysund",
-	   Calendar.MARCH, -1, Calendar.SUNDAY, 0 * 3600,
-	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 0 * 3600);
+	   Calendar.MARCH, -1, Calendar.SUNDAY, 1000 * 3600,
+	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 1000 * 3600);
 	timezones0.put("America/Scoresbysund", tz);
 	timezones0.put("Atlantic/Azores", tz);
 	tz = new SimpleTimeZone(-1000 * 3600, "Atlantic/Cape_Verde");
 	timezones0.put("Atlantic/Cape_Verde", tz);
-	timezones0.put("Atlantic/Jan_Mayen", tz);
 	tz = new SimpleTimeZone(0 * 3600, "GMT");
 	timezones0.put("GMT", tz);
 	timezones0.put("UTC", tz);
@@ -419,6 +440,7 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
 	timezones0.put("Africa/Ouagadougou", tz);
 	timezones0.put("Africa/Sao_Tome", tz);
 	timezones0.put("Africa/Timbuktu", tz);
+	timezones0.put("America/Danmarkshavn", tz);
 	timezones0.put("Atlantic/Reykjavik", tz);
 	timezones0.put("Atlantic/St_Helena", tz);
 	timezones0.put("Europe/Belfast", tz);
@@ -426,8 +448,8 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
 	timezones0.put("Europe/London", tz);
 	tz = new SimpleTimeZone
 	  (0 * 3600, "WET",
-	   Calendar.MARCH, -1, Calendar.SUNDAY, 1000 * 3600,
-	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 1000 * 3600);
+	   Calendar.MARCH, -1, Calendar.SUNDAY, 2000 * 3600,
+	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 2000 * 3600);
 	timezones0.put("WET", tz);
 	timezones0.put("Atlantic/Canary", tz);
 	timezones0.put("Atlantic/Faeroe", tz);
@@ -454,14 +476,14 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
 	timezones0.put("Africa/Windhoek", tz);
 	tz = new SimpleTimeZone
 	  (1000 * 3600, "CET",
-	   Calendar.MARCH, -1, Calendar.SUNDAY, 2000 * 3600,
-	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 2000 * 3600);
+	   Calendar.MARCH, -1, Calendar.SUNDAY, 3000 * 3600,
+	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 3000 * 3600);
 	timezones0.put("CET", tz);
-	timezones0.put("CEST", tz);
 	timezones0.put("ECT", tz);
 	timezones0.put("MET", tz);
 	timezones0.put("Africa/Ceuta", tz);
 	timezones0.put("Arctic/Longyearbyen", tz);
+	timezones0.put("Atlantic/Jan_Mayen", tz);
 	timezones0.put("Europe/Amsterdam", tz);
 	timezones0.put("Europe/Andorra", tz);
 	timezones0.put("Europe/Belgrade", tz);
@@ -493,8 +515,8 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
 	timezones0.put("Europe/Zurich", tz);
 	tz = new SimpleTimeZone
 	  (2000 * 3600, "ART",
-	   Calendar.APRIL, -1, Calendar.FRIDAY, 0 * 3600,
-	   Calendar.SEPTEMBER, -1, Calendar.THURSDAY, 23000 * 3600);
+	   Calendar.APRIL, -1, Calendar.FRIDAY, 1000 * 3600,
+	   Calendar.SEPTEMBER, -1, Calendar.THURSDAY, 24000 * 3600);
 	timezones0.put("ART", tz);
 	timezones0.put("Africa/Cairo", tz);
 	tz = new SimpleTimeZone(2000 * 3600, "CAT");
@@ -511,13 +533,11 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
 	timezones0.put("Africa/Maseru", tz);
 	timezones0.put("Africa/Mbabane", tz);
 	timezones0.put("Africa/Tripoli", tz);
-	timezones0.put("Europe/Riga", tz);
-	timezones0.put("Europe/Tallinn", tz);
-	timezones0.put("Europe/Vilnius", tz);
+	timezones0.put("Asia/Jerusalem", tz);
 	tz = new SimpleTimeZone
 	  (2000 * 3600, "Asia/Amman",
-	   Calendar.MARCH, -1, Calendar.THURSDAY, 0 * 3600,
-	   Calendar.SEPTEMBER, -1, Calendar.THURSDAY, 0 * 3600);
+	   Calendar.MARCH, -1, Calendar.THURSDAY, 1000 * 3600,
+	   Calendar.SEPTEMBER, -1, Calendar.THURSDAY, 1000 * 3600);
 	timezones0.put("Asia/Amman", tz);
 	tz = new SimpleTimeZone
 	  (2000 * 3600, "Asia/Beirut",
@@ -534,15 +554,10 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
 	   Calendar.APRIL, 3, Calendar.FRIDAY, 0 * 3600,
 	   Calendar.OCTOBER, 3, Calendar.FRIDAY, 0 * 3600);
 	timezones0.put("Asia/Gaza", tz);
-	tz = new SimpleTimeZone
-	  (2000 * 3600, "Asia/Jerusalem",
-	   Calendar.APRIL, 1, 0, 1000 * 3600,
-	   Calendar.OCTOBER, 1, 0, 1000 * 3600);
-	timezones0.put("Asia/Jerusalem", tz);
 	tz = new SimpleTimeZone
 	  (2000 * 3600, "EET",
-	   Calendar.MARCH, -1, Calendar.SUNDAY, 3000 * 3600,
-	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 3000 * 3600);
+	   Calendar.MARCH, -1, Calendar.SUNDAY, 4000 * 3600,
+	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 4000 * 3600);
 	timezones0.put("EET", tz);
 	timezones0.put("Asia/Istanbul", tz);
 	timezones0.put("Asia/Nicosia", tz);
@@ -552,28 +567,32 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
 	timezones0.put("Europe/Helsinki", tz);
 	timezones0.put("Europe/Istanbul", tz);
 	timezones0.put("Europe/Kiev", tz);
+	timezones0.put("Europe/Mariehamn", tz);
 	timezones0.put("Europe/Nicosia", tz);
+	timezones0.put("Europe/Riga", tz);
 	timezones0.put("Europe/Simferopol", tz);
 	timezones0.put("Europe/Sofia", tz);
+	timezones0.put("Europe/Tallinn", tz);
 	timezones0.put("Europe/Uzhgorod", tz);
+	timezones0.put("Europe/Vilnius", tz);
 	timezones0.put("Europe/Zaporozhye", tz);
 	tz = new SimpleTimeZone
 	  (2000 * 3600, "Europe/Kaliningrad",
-	   Calendar.MARCH, -1, Calendar.SUNDAY, 2000 * 3600,
-	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 2000 * 3600);
+	   Calendar.MARCH, -1, Calendar.SUNDAY, 3000 * 3600,
+	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 3000 * 3600);
 	timezones0.put("Europe/Kaliningrad", tz);
 	timezones0.put("Europe/Minsk", tz);
 	tz = new SimpleTimeZone
 	  (3000 * 3600, "Asia/Baghdad",
-	   Calendar.APRIL, 1, 0, 3000 * 3600,
-	   Calendar.OCTOBER, 1, 0, 3000 * 3600);
+	   Calendar.APRIL, 1, 0, 4000 * 3600,
+	   Calendar.OCTOBER, 1, 0, 4000 * 3600);
 	timezones0.put("Asia/Baghdad", tz);
 	tz = new SimpleTimeZone
-	  (3000 * 3600, "Europe/Moscow",
-	   Calendar.MARCH, -1, Calendar.SUNDAY, 2000 * 3600,
-	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 2000 * 3600);
+	  (3000 * 3600, "Asia/Tbilisi",
+	   Calendar.MARCH, -1, Calendar.SUNDAY, 3000 * 3600,
+	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 3000 * 3600);
+	timezones0.put("Asia/Tbilisi", tz);
 	timezones0.put("Europe/Moscow", tz);
-	timezones0.put("Europe/Tiraspol", tz);
 	tz = new SimpleTimeZone(3000 * 3600, "EAT");
 	timezones0.put("EAT", tz);
 	timezones0.put("Africa/Addis_Ababa", tz);
@@ -600,32 +619,23 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
 	   Calendar.MARCH, -1, Calendar.SUNDAY, 1000 * 3600,
 	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 1000 * 3600);
 	timezones0.put("Asia/Baku", tz);
-	tz = new SimpleTimeZone
-	  (4000 * 3600, "Asia/Aqtau",
-	   Calendar.MARCH, -1, Calendar.SUNDAY, 0 * 3600,
-	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 0 * 3600);
-	timezones0.put("Asia/Aqtau", tz);
-	timezones0.put("Asia/Tbilisi", tz);
 	tz = new SimpleTimeZone
 	  (4000 * 3600, "Asia/Yerevan",
-	   Calendar.MARCH, -1, Calendar.SUNDAY, 2000 * 3600,
-	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 2000 * 3600);
+	   Calendar.MARCH, -1, Calendar.SUNDAY, 3000 * 3600,
+	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 3000 * 3600);
 	timezones0.put("Asia/Yerevan", tz);
 	timezones0.put("Europe/Samara", tz);
 	tz = new SimpleTimeZone(4000 * 3600, "NET");
 	timezones0.put("NET", tz);
+	timezones0.put("Asia/Aqtau", tz);
 	timezones0.put("Asia/Dubai", tz);
 	timezones0.put("Asia/Muscat", tz);
+	timezones0.put("Asia/Oral", tz);
 	timezones0.put("Indian/Mahe", tz);
 	timezones0.put("Indian/Mauritius", tz);
 	timezones0.put("Indian/Reunion", tz);
 	tz = new SimpleTimeZone(4500 * 3600, "Asia/Kabul");
 	timezones0.put("Asia/Kabul", tz);
-	tz = new SimpleTimeZone
-	  (5000 * 3600, "Asia/Aqtobe",
-	   Calendar.MARCH, -1, Calendar.SUNDAY, 0 * 3600,
-	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 0 * 3600);
-	timezones0.put("Asia/Aqtobe", tz);
 	tz = new SimpleTimeZone
 	  (5000 * 3600, "Asia/Bishkek",
 	   Calendar.MARCH, -1, Calendar.SUNDAY, 2500 * 3600,
@@ -633,17 +643,17 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
 	timezones0.put("Asia/Bishkek", tz);
 	tz = new SimpleTimeZone
 	  (5000 * 3600, "Asia/Yekaterinburg",
-	   Calendar.MARCH, -1, Calendar.SUNDAY, 2000 * 3600,
-	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 2000 * 3600);
+	   Calendar.MARCH, -1, Calendar.SUNDAY, 3000 * 3600,
+	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 3000 * 3600);
 	timezones0.put("Asia/Yekaterinburg", tz);
 	tz = new SimpleTimeZone(5000 * 3600, "PLT");
 	timezones0.put("PLT", tz);
+	timezones0.put("Asia/Aqtobe", tz);
 	timezones0.put("Asia/Ashgabat", tz);
 	timezones0.put("Asia/Dushanbe", tz);
 	timezones0.put("Asia/Karachi", tz);
 	timezones0.put("Asia/Samarkand", tz);
 	timezones0.put("Asia/Tashkent", tz);
-	timezones0.put("Indian/Chagos", tz);
 	timezones0.put("Indian/Kerguelen", tz);
 	timezones0.put("Indian/Maldives", tz);
 	tz = new SimpleTimeZone(5500 * 3600, "IST");
@@ -654,18 +664,17 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
 	tz = new SimpleTimeZone(6000 * 3600, "BST");
 	timezones0.put("BST", tz);
 	timezones0.put("Antarctica/Mawson", tz);
+	timezones0.put("Antarctica/Vostok", tz);
+	timezones0.put("Asia/Almaty", tz);
 	timezones0.put("Asia/Colombo", tz);
 	timezones0.put("Asia/Dhaka", tz);
+	timezones0.put("Asia/Qyzylorda", tz);
 	timezones0.put("Asia/Thimphu", tz);
-	tz = new SimpleTimeZone
-	  (6000 * 3600, "Asia/Almaty",
-	   Calendar.MARCH, -1, Calendar.SUNDAY, 0 * 3600,
-	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 0 * 3600);
-	timezones0.put("Asia/Almaty", tz);
+	timezones0.put("Indian/Chagos", tz);
 	tz = new SimpleTimeZone
 	  (6000 * 3600, "Asia/Novosibirsk",
-	   Calendar.MARCH, -1, Calendar.SUNDAY, 2000 * 3600,
-	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 2000 * 3600);
+	   Calendar.MARCH, -1, Calendar.SUNDAY, 3000 * 3600,
+	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 3000 * 3600);
 	timezones0.put("Asia/Novosibirsk", tz);
 	timezones0.put("Asia/Omsk", tz);
 	tz = new SimpleTimeZone(6500 * 3600, "Asia/Rangoon");
@@ -675,41 +684,55 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
 	timezones0.put("VST", tz);
 	timezones0.put("Antarctica/Davis", tz);
 	timezones0.put("Asia/Bangkok", tz);
-	timezones0.put("Asia/Hovd", tz);
 	timezones0.put("Asia/Jakarta", tz);
 	timezones0.put("Asia/Phnom_Penh", tz);
+	timezones0.put("Asia/Pontianak", tz);
 	timezones0.put("Asia/Saigon", tz);
 	timezones0.put("Asia/Vientiane", tz);
 	timezones0.put("Indian/Christmas", tz);
+	tz = new SimpleTimeZone
+	  (7000 * 3600, "Asia/Hovd",
+	   Calendar.MARCH, -1, Calendar.SATURDAY, 2000 * 3600,
+	   Calendar.SEPTEMBER, -1, Calendar.SATURDAY, 2000 * 3600);
+	timezones0.put("Asia/Hovd", tz);
 	tz = new SimpleTimeZone
 	  (7000 * 3600, "Asia/Krasnoyarsk",
-	   Calendar.MARCH, -1, Calendar.SUNDAY, 2000 * 3600,
-	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 2000 * 3600);
+	   Calendar.MARCH, -1, Calendar.SUNDAY, 3000 * 3600,
+	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 3000 * 3600);
 	timezones0.put("Asia/Krasnoyarsk", tz);
 	tz = new SimpleTimeZone(8000 * 3600, "CTT");
 	timezones0.put("CTT", tz);
 	timezones0.put("Antarctica/Casey", tz);
 	timezones0.put("Asia/Brunei", tz);
-	timezones0.put("Asia/Chungking", tz);
+	timezones0.put("Asia/Chongqing", tz);
 	timezones0.put("Asia/Harbin", tz);
 	timezones0.put("Asia/Hong_Kong", tz);
 	timezones0.put("Asia/Kashgar", tz);
 	timezones0.put("Asia/Kuala_Lumpur", tz);
 	timezones0.put("Asia/Kuching", tz);
-	timezones0.put("Asia/Macao", tz);
+	timezones0.put("Asia/Macau", tz);
+	timezones0.put("Asia/Makassar", tz);
 	timezones0.put("Asia/Manila", tz);
 	timezones0.put("Asia/Shanghai", tz);
 	timezones0.put("Asia/Singapore", tz);
 	timezones0.put("Asia/Taipei", tz);
-	timezones0.put("Asia/Ujung_Pandang", tz);
-	timezones0.put("Asia/Ulaanbaatar", tz);
 	timezones0.put("Asia/Urumqi", tz);
 	timezones0.put("Australia/Perth", tz);
 	tz = new SimpleTimeZone
 	  (8000 * 3600, "Asia/Irkutsk",
-	   Calendar.MARCH, -1, Calendar.SUNDAY, 2000 * 3600,
-	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 2000 * 3600);
+	   Calendar.MARCH, -1, Calendar.SUNDAY, 3000 * 3600,
+	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 3000 * 3600);
 	timezones0.put("Asia/Irkutsk", tz);
+	tz = new SimpleTimeZone
+	  (8000 * 3600, "Asia/Ulaanbaatar",
+	   Calendar.MARCH, -1, Calendar.SATURDAY, 2000 * 3600,
+	   Calendar.SEPTEMBER, -1, Calendar.SATURDAY, 2000 * 3600);
+	timezones0.put("Asia/Ulaanbaatar", tz);
+	tz = new SimpleTimeZone
+	  (9000 * 3600, "Asia/Choibalsan",
+	   Calendar.MARCH, -1, Calendar.SATURDAY, 2000 * 3600,
+	   Calendar.SEPTEMBER, -1, Calendar.SATURDAY, 2000 * 3600);
+	timezones0.put("Asia/Choibalsan", tz);
 	tz = new SimpleTimeZone(9000 * 3600, "JST");
 	timezones0.put("JST", tz);
 	timezones0.put("Asia/Dili", tz);
@@ -720,13 +743,13 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
 	timezones0.put("Pacific/Palau", tz);
 	tz = new SimpleTimeZone
 	  (9000 * 3600, "Asia/Yakutsk",
-	   Calendar.MARCH, -1, Calendar.SUNDAY, 2000 * 3600,
-	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 2000 * 3600);
+	   Calendar.MARCH, -1, Calendar.SUNDAY, 3000 * 3600,
+	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 3000 * 3600);
 	timezones0.put("Asia/Yakutsk", tz);
 	tz = new SimpleTimeZone
 	  (9500 * 3600, "Australia/Adelaide",
-	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 2000 * 3600,
-	   Calendar.MARCH, -1, Calendar.SUNDAY, 2000 * 3600);
+	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 3000 * 3600,
+	   Calendar.MARCH, -1, Calendar.SUNDAY, 3000 * 3600);
 	timezones0.put("Australia/Adelaide", tz);
 	timezones0.put("Australia/Broken_Hill", tz);
 	tz = new SimpleTimeZone(9500 * 3600, "ACT");
@@ -742,31 +765,32 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
 	timezones0.put("Pacific/Truk", tz);
 	timezones0.put("Pacific/Yap", tz);
 	tz = new SimpleTimeZone
-	  (10000 * 3600, "Asia/Vladivostok",
-	   Calendar.MARCH, -1, Calendar.SUNDAY, 2000 * 3600,
-	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 2000 * 3600);
+	  (10000 * 3600, "Asia/Sakhalin",
+	   Calendar.MARCH, -1, Calendar.SUNDAY, 3000 * 3600,
+	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 3000 * 3600);
+	timezones0.put("Asia/Sakhalin", tz);
 	timezones0.put("Asia/Vladivostok", tz);
 	tz = new SimpleTimeZone
 	  (10000 * 3600, "Australia/Hobart",
-	   Calendar.OCTOBER, 1, Calendar.SUNDAY, 2000 * 3600,
-	   Calendar.MARCH, -1, Calendar.SUNDAY, 2000 * 3600);
+	   Calendar.OCTOBER, 1, Calendar.SUNDAY, 3000 * 3600,
+	   Calendar.MARCH, -1, Calendar.SUNDAY, 3000 * 3600);
 	timezones0.put("Australia/Hobart", tz);
 	tz = new SimpleTimeZone
 	  (10000 * 3600, "AET",
-	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 2000 * 3600,
-	   Calendar.MARCH, -1, Calendar.SUNDAY, 2000 * 3600);
+	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 3000 * 3600,
+	   Calendar.MARCH, -1, Calendar.SUNDAY, 3000 * 3600);
 	timezones0.put("AET", tz);
 	timezones0.put("Australia/Melbourne", tz);
 	timezones0.put("Australia/Sydney", tz);
 	tz = new SimpleTimeZone
 	  (10500 * 3600, "Australia/Lord_Howe",
-	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 2000 * 3600,
-	   Calendar.MARCH, -1, Calendar.SUNDAY, 2000 * 3600, 500 * 3600);
+	  Calendar.OCTOBER, -1, Calendar.SUNDAY, 2000 * 3600,
+	  Calendar.MARCH, -1, Calendar.SUNDAY, 2000 * 3600, 500 * 3600);
 	timezones0.put("Australia/Lord_Howe", tz);
 	tz = new SimpleTimeZone
 	  (11000 * 3600, "Asia/Magadan",
-	   Calendar.MARCH, -1, Calendar.SUNDAY, 2000 * 3600,
-	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 2000 * 3600);
+	   Calendar.MARCH, -1, Calendar.SUNDAY, 3000 * 3600,
+	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 3000 * 3600);
 	timezones0.put("Asia/Magadan", tz);
 	tz = new SimpleTimeZone(11000 * 3600, "SST");
 	timezones0.put("SST", tz);
@@ -779,16 +803,16 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
 	timezones0.put("Pacific/Norfolk", tz);
 	tz = new SimpleTimeZone
 	  (12000 * 3600, "NST",
-	   Calendar.OCTOBER, 1, Calendar.SUNDAY, 2000 * 3600,
-	   Calendar.MARCH, 3, Calendar.SUNDAY, 2000 * 3600);
+	   Calendar.OCTOBER, 1, Calendar.SUNDAY, 3000 * 3600,
+	   Calendar.MARCH, 3, Calendar.SUNDAY, 3000 * 3600);
 	timezones0.put("NST", tz);
 	timezones0.put("Antarctica/McMurdo", tz);
 	timezones0.put("Antarctica/South_Pole", tz);
 	timezones0.put("Pacific/Auckland", tz);
 	tz = new SimpleTimeZone
 	  (12000 * 3600, "Asia/Anadyr",
-	   Calendar.MARCH, -1, Calendar.SUNDAY, 2000 * 3600,
-	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 2000 * 3600);
+	   Calendar.MARCH, -1, Calendar.SUNDAY, 3000 * 3600,
+	   Calendar.OCTOBER, -1, Calendar.SUNDAY, 3000 * 3600);
 	timezones0.put("Asia/Anadyr", tz);
 	timezones0.put("Asia/Kamchatka", tz);
 	tz = new SimpleTimeZone(12000 * 3600, "Pacific/Fiji");
@@ -802,8 +826,8 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
 	timezones0.put("Pacific/Wallis", tz);
 	tz = new SimpleTimeZone
 	  (12750 * 3600, "Pacific/Chatham",
-	   Calendar.OCTOBER, 1, Calendar.SUNDAY, 2750 * 3600,
-	   Calendar.MARCH, 3, Calendar.SUNDAY, 2750 * 3600);
+	   Calendar.OCTOBER, 1, Calendar.SUNDAY, 3750 * 3600,
+	   Calendar.MARCH, 3, Calendar.SUNDAY, 3750 * 3600);
 	timezones0.put("Pacific/Chatham", tz);
 	tz = new SimpleTimeZone(13000 * 3600, "Pacific/Enderbury");
 	timezones0.put("Pacific/Enderbury", tz);
@@ -820,13 +844,20 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
    * with the result of <code>System.getProperty("user.timezone")</code>
    * or <code>getDefaultTimeZoneId()</code>.  Note that giving one of
    * the standard tz data names from ftp://elsie.nci.nih.gov/pub/ is
-   * preferred.  The time zone name can be given as follows:
-   * <code>(standard zone name)[(GMT offset)[(daylight time zone name)]]</code>
+   * preferred.  
+   * The time zone name can be given as follows:
+   * <code>(standard zone name)[(GMT offset)[(DST zone name)[DST offset]]]
+   * </code>
    * <p>
    * If only a (standard zone name) is given (no numbers in the
    * String) then it gets mapped directly to the TimeZone with that
    * name, if that fails null is returned.
    * <p>
+   * Alternately, a POSIX-style TZ string can be given, defining the time zone:
+   * <code>std offset dst offset,date/time,date/time</code>
+   * See the glibc manual, or the man page for <code>tzset</code> for details
+   * of this format.
+   * <p>
    * A GMT offset is the offset to add to the local time to get GMT.
    * If a (GMT offset) is included (either in seconds or hours) then
    * an attempt is made to find a TimeZone name matching both the name
@@ -853,95 +884,280 @@ public abstract class TimeZone implements java.io.Serializable, Cloneable
    */
   static TimeZone getDefaultTimeZone(String sysTimeZoneId)
   {
-    // First find start of GMT offset info and any Daylight zone name.
-    int startGMToffset = 0;
-    int sysTimeZoneIdLength = sysTimeZoneId.length();
-    for (int i = 0; i < sysTimeZoneIdLength && startGMToffset == 0; i++)
+    String stdName = null;
+    String dstName;
+    int stdOffs;
+    int dstOffs;
+    try
       {
-	char c = sysTimeZoneId.charAt(i);
-	if (Character.isDigit(c))
-	  startGMToffset = i;
-	else if ((c == '+' || c == '-')
-		 && i + 1 < sysTimeZoneIdLength
-		 && Character.isDigit(sysTimeZoneId.charAt(i + 1)))
-	  startGMToffset = i;
-      }
-    
-    String tzBasename;
-    if (startGMToffset == 0)
-      tzBasename = sysTimeZoneId;
-    else
-      tzBasename = sysTimeZoneId.substring (0, startGMToffset);
-    
-    int startDaylightZoneName = 0;
-    for (int i = sysTimeZoneIdLength - 1;
-	 i >= 0 && !Character.isDigit(sysTimeZoneId.charAt(i)); --i)
-      startDaylightZoneName = i;
-    
-    boolean useDaylightTime = startDaylightZoneName > 0;
-    
-    // Integer.parseInt() doesn't handle leading +.
-    if (sysTimeZoneId.charAt(startGMToffset) == '+')
-      startGMToffset++;
-    
-    int gmtOffset = 0;
-    if (startGMToffset > 0)
-      {
-	gmtOffset = Integer.parseInt
-	  (startDaylightZoneName == 0
-	   ? sysTimeZoneId.substring(startGMToffset)
-	   : sysTimeZoneId.substring(startGMToffset,
-				     startDaylightZoneName));
-	
-	// Offset could be in hours or seconds.  Convert to millis.
-	// The offset is given as the time to add to local time to get GMT
-	// we need the time to add to GMT to get localtime.
-	if (Math.abs(gmtOffset) < 24)
-	  gmtOffset *= 60 * 60;
-	gmtOffset *= -1000;
-      }
-    
-    // Try to be optimistic and get the timezone that matches the base name.
-    // If we only have the base name then just accept this timezone.
-    // Otherwise check the gmtOffset and day light attributes.
-    TimeZone tz = (TimeZone) timezones().get(tzBasename);
-    if (tz != null
-	&& (tzBasename == sysTimeZoneId
-	    || (tz.getRawOffset() == gmtOffset
-		&& tz.useDaylightTime() == useDaylightTime)))
-      return tz;
-    
-    // Maybe there is one with the daylight zone name?
-    if (useDaylightTime)
-      {
-	String daylightZoneName;
-	daylightZoneName = sysTimeZoneId.substring(startDaylightZoneName);
-	if (!daylightZoneName.equals(tzBasename))
+	int idLength = sysTimeZoneId.length();
+
+	int index = 0;
+	int prevIndex;
+	char c;
+
+	// get std
+	do
+	  c = sysTimeZoneId.charAt(index++);
+	while (c != '+' && c != '-' && c != ',' && c != ':'
+	       && ! Character.isDigit(c) && c != '\0' && index < idLength);
+
+	if (index >= idLength)
+	  return (TimeZone)timezones().get(sysTimeZoneId);
+
+	stdName = sysTimeZoneId.substring(0, --index);
+	prevIndex = index;
+
+	// get the std offset
+	do
+	  c = sysTimeZoneId.charAt(index++);
+	while ((c == '-' || c == '+' || c == ':' || Character.isDigit(c))
+	       && index < idLength);
+	if (index < idLength)
+	  index--;
+
+	{ // convert the dst string to a millis number
+	    String offset = sysTimeZoneId.substring(prevIndex, index);
+	    prevIndex = index;
+
+	    if (offset.charAt(0) == '+' || offset.charAt(0) == '-')
+	      stdOffs = parseTime(offset.substring(1));
+	    else
+	      stdOffs = parseTime(offset);
+
+	    if (offset.charAt(0) == '-')
+	      stdOffs = -stdOffs;
+
+	    // TZ timezone offsets are positive when WEST of the meridian.
+	    stdOffs = -stdOffs;
+	}
+
+	// Done yet? (Format: std offset)
+	if (index >= idLength)
 	  {
-	    tz = (TimeZone) timezones().get(tzBasename);
-	    if (tz != null
-		&& tz.getRawOffset() == gmtOffset
-		&& tz.useDaylightTime())
-	      return tz;
+	    // Do we have an existing timezone with that name and offset?
+	    TimeZone tz = (TimeZone) timezones().get(stdName);
+	    if (tz != null)
+	      if (tz.getRawOffset() == stdOffs)
+		return tz;
+
+	    // Custom then.
+	    return new SimpleTimeZone(stdOffs, stdName);
+	  }
+
+	// get dst
+	do
+	  c = sysTimeZoneId.charAt(index++);
+	while (c != '+' && c != '-' && c != ',' && c != ':'
+	       && ! Character.isDigit(c) && c != '\0' && index < idLength);
+
+	// Done yet? (Format: std offset dst)
+	if (index >= idLength)
+	  {
+	    // Do we have an existing timezone with that name and offset 
+	    // which has DST?
+	    TimeZone tz = (TimeZone) timezones().get(stdName);
+	    if (tz != null)
+	      if (tz.getRawOffset() == stdOffs && tz.useDaylightTime())
+		return tz;
+
+	    // Custom then.
+	    return new SimpleTimeZone(stdOffs, stdName);
+	  }
+
+	// get the dst offset
+	dstName = sysTimeZoneId.substring(prevIndex, --index);
+	prevIndex = index;
+	do
+	  c = sysTimeZoneId.charAt(index++);
+	while ((c == '-' || c == '+' || c == ':' || Character.isDigit(c))
+	       && index < idLength);
+	if (index < idLength)
+	  index--;
+
+	{ // convert the dst string to a millis number
+	    String offset = sysTimeZoneId.substring(prevIndex, index);
+	    prevIndex = index;
+
+	    if (offset.charAt(0) == '+' || offset.charAt(0) == '-')
+	      dstOffs = parseTime(offset.substring(1));
+	    else
+	      dstOffs = parseTime(offset);
+
+	    if (offset.charAt(0) == '-')
+	      dstOffs = -dstOffs;
+
+	    // TZ timezone offsets are positive when WEST of the meridian.
+	    dstOffs = -dstOffs;
+	}
+
+	// Done yet? (Format: std offset dst offset)
+	// FIXME: We don't support DST without a rule given. Should we?
+	if (index >= idLength)
+	  {
+	    // Time Zone existing with same name, dst and offsets?
+	    TimeZone tz = (TimeZone) timezones().get(stdName);
+	    if (tz != null)
+	      if (tz.getRawOffset() == stdOffs && tz.useDaylightTime()
+	          && tz.getDSTSavings() == (dstOffs - stdOffs))
+		return tz;
+
+	    return new SimpleTimeZone(stdOffs, stdName);
+	  }
+
+	// get the DST rule
+	if (sysTimeZoneId.charAt(index) == ','
+	    || sysTimeZoneId.charAt(index) == ';')
+	  {
+	    index++;
+	    int offs = index;
+	    while (sysTimeZoneId.charAt(index) != ','
+	           && sysTimeZoneId.charAt(index) != ';')
+	      index++;
+	    String startTime = sysTimeZoneId.substring(offs, index);
+	    index++;
+	    String endTime = sysTimeZoneId.substring(index);
+
+	    index = startTime.indexOf('/');
+	    int startMillis;
+	    int endMillis;
+	    String startDate;
+	    String endDate;
+	    if (index != -1)
+	      {
+		startDate = startTime.substring(0, index);
+		startMillis = parseTime(startTime.substring(index + 1));
+	      }
+	    else
+	      {
+		startDate = startTime;
+		// if time isn't given, default to 2:00:00 AM.
+		startMillis = 2 * 60 * 60 * 1000;
+	      }
+	    index = endTime.indexOf('/');
+	    if (index != -1)
+	      {
+		endDate = endTime.substring(0, index);
+		endMillis = parseTime(endTime.substring(index + 1));
+	      }
+	    else
+	      {
+		endDate = endTime;
+		// if time isn't given, default to 2:00:00 AM.
+		endMillis = 2 * 60 * 60 * 1000;
+	      }
+
+	    int[] start = getDateParams(startDate);
+	    int[] end = getDateParams(endDate);
+	    return new SimpleTimeZone(stdOffs, stdName, start[0], start[1],
+	                              start[2], startMillis, end[0], end[1],
+	                              end[2], endMillis, (dstOffs - stdOffs));
 	  }
       }
-    
-    // If no match, see if a valid timezone has similar attributes as this
-    // and then use it instead. We take the first one that looks OKish.
-    if (startGMToffset > 0)
+
+    // FIXME: Produce a warning here?
+    catch (IndexOutOfBoundsException _)
       {
-	String[] ids = getAvailableIDs(gmtOffset);
-	for (int i = 0; i < ids.length; i++)
-	  {
-	    tz = (TimeZone) timezones().get(ids[i]);
-	    if (tz.useDaylightTime() == useDaylightTime)
-	      return tz;
-	  }
       }
-    
+    catch (NumberFormatException _)
+      {
+      }
+
     return null;
   }
 
+  /**
+   * Parses and returns the params for a POSIX TZ date field,
+   * in the format int[]{ month, day, dayOfWeek }, following the
+   * SimpleTimeZone constructor rules.
+   */
+  private static int[] getDateParams(String date)
+  {
+    int[] dayCount = { 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334 };
+    int month;
+
+    if (date.charAt(0) == 'M' || date.charAt(0) == 'm')
+      {
+	int day;
+
+	// Month, week of month, day of week
+	month = Integer.parseInt(date.substring(1, date.indexOf('.')));
+	int week = Integer.parseInt(date.substring(date.indexOf('.') + 1,
+	                                           date.lastIndexOf('.')));
+	int dayOfWeek = Integer.parseInt(date.substring(date.lastIndexOf('.')
+	                                                + 1));
+	if (week == 5)
+	  day = -1; // last day of month is -1 in java, 5 in TZ
+	else
+	  // first day of week starting on or after.
+	  day = (week - 1) * 7 + 1;
+
+	dayOfWeek++; // Java day of week is one-based, Sunday is first day.
+	month--; // Java month is zero-based.
+	return new int[] { month, day, dayOfWeek };
+      }
+
+    // julian day, either zero-based 0<=n<=365 (incl feb 29)
+    // or one-based 1<=n<=365 (no feb 29)
+    int julianDay; // Julian day, 
+
+    if (date.charAt(0) != 'J' || date.charAt(0) != 'j')
+      {
+	julianDay = Integer.parseInt(date.substring(1));
+	julianDay++; // make 1-based
+	// Adjust day count to include feb 29.
+	dayCount = new int[]
+	           {
+	             0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335
+	           };
+      }
+    else
+      // 1-based julian day
+      julianDay = Integer.parseInt(date);
+
+    int i = 11;
+    while (i > 0)
+      if (dayCount[i] < julianDay)
+	break;
+      else
+	i--;
+    julianDay -= dayCount[i];
+    month = i;
+    return new int[] { month, julianDay, 0 };
+  }
+
+  /**
+   * Parses a time field hh[:mm[:ss]], returning the result
+   * in milliseconds. No leading sign.
+   */
+  private static int parseTime(String time)
+  {
+    int millis = 0;
+    int i = 0;
+
+    while (i < time.length())
+      if (time.charAt(i) == ':')
+	break;
+      else
+	i++;
+    millis = 60 * 60 * 1000 * Integer.parseInt(time.substring(0, i));
+    if (i >= time.length())
+      return millis;
+
+    int iprev = ++i;
+    while (i < time.length())
+      if (time.charAt(i) == ':')
+	break;
+      else
+	i++;
+    if (i >= time.length())
+      return millis;
+
+    millis += 60 * 1000 * Integer.parseInt(time.substring(iprev, i));
+    millis += 1000 * Integer.parseInt(time.substring(++i));
+    return millis;
+  }
+
   /**
    * Gets the time zone offset, for current date, modified in case of 
    * daylight savings.  This is the offset to add to UTC to get the local
