@@ -19,8 +19,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -194,11 +194,18 @@ final class VMClass
    * public and final, but not an interface.
    *
    * @param klass the Class object that's calling us
+   * @param ignoreInnerClassesAttrib if set, return the real modifiers, not
+   * the ones specified in the InnerClasses attribute.
    * @return the modifiers of this class
    * @see Modifer
    * @since 1.1
    */
-  static native int getModifiers(Class klass);
+  static int getModifiers(Class klass, boolean ignoreInnerClassesAttrib)
+  {
+    return getModifiers(klass);
+  }
+
+  private static native int getModifiers(Class klass);
 
   /**
    * If this is a nested or inner class, return the class that declared it.
@@ -252,13 +259,15 @@ final class VMClass
   static native ClassLoader getClassLoader(Class klass);
 
   /**
-   * VM implementors are free to make this method a noop if 
-   * the default implementation is acceptable.
+   * Load the requested class and record the specified loader as the
+   * initiating class loader.
    *
    * @param name the name of the class to find
+   * @param initialize should the class initializer be run?
+   * @param loader the class loader to use (or null for the bootstrap loader)
    * @return the Class object representing the class or null for noop
    * @throws ClassNotFoundException if the class was not found by the
-   *         classloader
+   *         class loader
    * @throws LinkageError if linking the class fails
    * @throws ExceptionInInitializerError if the class loads, but an exception
    *         occurs during initialization
@@ -268,6 +277,13 @@ final class VMClass
     return null;
   }
 
+  static Class forName(String name, boolean initialize,
+                              ClassLoader loader)
+    throws ClassNotFoundException
+  {
+    return null;
+  }
+
   /**
    * Return whether this class is an array type.
    *
